"""Proxylists Meta information."""

__title__ = 'proxylists'
__description__ = ('Package for getting useful proxy servers, '
                   'can use lists like hidemy or proxydb.')
__version__ = '0.12.2'
__author__ = 'Jesus Lara'
__author_email__ = 'jesuslarag@gmail.com'
__license__ = 'BSD'



import aiohttp
import asyncio
from lxml.html import fromstring
from abc import abstractmethod


class ProxyServer(object):
    url = ""
    parser = None

    async def get_list(self):
        timeout = aiohttp.ClientTimeout(total=60)
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "Accept-Encoding": "gzip, deflate",
            "Dnt": "1",
            "Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) "
            "AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/83.0.4103.97 Safari/537.36",
        }
        async with aiohttp.ClientSession() as session:
            async with session.get(self.url, headers=headers, timeout=timeout) as response:
                if response.status == 200:
                    try:
                        content = await response.text()
                    except Exception as e:
                        print(e)
                        b = await response.content.read()
                        content = b.decode("utf-8")
                    self.parser = fromstring(content)
                    return await self.get_proxies()

    @abstractmethod
    async def get_proxies(self):
        pass



import logging
from .server import ProxyServer



class Hidemy(ProxyServer):
    url = "https://hidemy.name/es/proxy-list/?type=s#list"

    async def get_proxies(self):
        proxies = []
        try:
            table = self.parser.xpath("//table")[0]
        except Exception as err:
            logging.exception(err)
            return []
        for i in table.xpath("//tbody/tr")[:10]:
            if i.xpath('.//td[5][contains(text(),"HTTPS")]'):
                proxy = ":".join(
                    [i.xpath(".//td[1]/text()")[0], i.xpath(".//td[2]/text()")[0]]
                )
                proxies.append(proxy)
        return proxies



import socket
import asyncio


def check_host(host: str, port: int = 80, timeout: int = 2) -> bool:
    """
    Check if an Address is reachable and available.
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.settimeout(timeout)  # timeout
        s.connect((host, int(port)))
        return True
    except:
        return False
    finally:
      s.shutdown(socket.SHUT_RDWR)
      s.close()


async def check_address(host: str, port: int = 80, timeout: int = 2) -> bool:
    """
    Async version of test if an address (IP) is reachable.
    Parameters:
      host: str
        host IP address or hostname
      port : int
        HTTP port number
    Returns
    -------
    awaitable bool
    """
    try:
        reader, writer = await asyncio.wait_for(
            asyncio.open_connection(host, port),
            timeout=timeout
        )
        writer.close()
        await writer.wait_closed()
        return True
    except:
        return False



"""ProxyLists.

    Get a list of free Proxy Servers
See:
https://github.com/phenobarbital/proxylists
"""
import ast
from os import path
from setuptools import setup, find_packages

def get_path(filename):
    return path.join(path.dirname(path.abspath(__file__)), filename)


def readme():
    with open(get_path('README.md'), encoding='utf-8') as rd:
        return rd.read()


version = get_path('proxylists/version.py')
with open(version, 'r', encoding='utf-8') as meta:
    # exec(meta.read())
    t = compile(meta.read(), version, 'exec', ast.PyCF_ONLY_AST)
    for node in (n for n in t.body if isinstance(n, ast.Assign)):
        if len(node.targets) == 1:
            name = node.targets[0]
            if isinstance(name, ast.Name) and \
                    name.id in (
                        '__version__',
                        '__title__',
                        '__description__',
                        '__author__',
                        '__license__', '__author_email__'):
                v = node.value
                if name.id == '__version__':
                    __version__ = v.s
                if name.id == '__title__':
                    __title__ = v.s
                if name.id == '__description__':
                    __description__ = v.s
                if name.id == '__license__':
                    __license__ = v.s
                if name.id == '__author__':
                    __author__ = v.s
                if name.id == '__author_email__':
                    __author_email__ = v.s


setup(
    name='proxylists',
    version=__version__,
    python_requires=">=3.8.0",
    url='https://github.com/phenobarbital/proxylists',
    description=__description__,
    long_description=readme(),
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
    ],
    author=__author__,
    author_email=__author_email__,
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    license=__license__,
    license_file='LICENSE',
    setup_requires=[
        "asyncio==3.4.3"
    ],
    install_requires=[
        "asyncio==3.4.3",
        "uvloop>=0.19.0",
        "aiohttp==3.9.1",
        'requests>=2.28.2',
        'requests[socks]>=2.28.2',
        'orjson==3.9.10',
        'lxml>=4.6.0'
    ],
    tests_require=[
        'pytest>=5.4.0',
        'coverage'
    ],
    project_urls={  # Optional
        'Source': 'https://github.com/phenobarbital/proxylists',
        'Funding': 'https://paypal.me/phenobarbital',
        'Say Thanks!': 'https://saythanks.io/to/phenobarbital',
    },
)



import asyncio
from .proxies import PROXY_LIST
from .proxies.connections import check_host, check_address


__all__ = ["PROXY_LIST", "check_host", "check_address"]


async def proxy_list():
    proxies = []
    for proxy in PROXY_LIST:
        p = await proxy().get_list()
        try:
            proxies = proxies + p
        except TypeError:
            continue
    return proxies


def get_proxies():
    loop = asyncio.get_event_loop()
    return loop.run_until_complete(proxy_list())
